%%% Econ 203C (Spring 2009)
%%% Problem Set 3, Questions 2 and 3
%%% Monte Carlo Simulation of Wald and LMcue statistics weak instrumental variables

clear all;              % clear all the variables
clc;                    % clear the screen

% Set primitive variables
R=1000;                 % number of Monte Carlo simulation
n=100;                  % sample size in each simulation
beta=[-.8:.2:.8];       % true parameters to be estimated
K=[1,10];               % number of instrumental variables
eta=[.05,1];            % strength of instruments
rho=[.5,.99];           % endogeneity of regressor
beta0=0;                % the null hypothesis

% Initialize the sequence of random numbers
randn('seed',2032009);

% Initialize the estimates matrix
beta_est = [];
Wald_sim = [];
LMcue_sim = [];

% Start iteration
for r=1:R
    % Initialize the intermediate estimates matrix
    beta_temp = [];
    Wald_temp = [];
    LMcue_temp = [];
    
    for h=1:length(K)
        % Generate random matrix used through all the eta's and the rho's
        V = randn(n,2);             % standard normal random vector with var=I
        Z = randn(n,K(h));          % standard normal random vector with var=I
        
        for i=1:length(eta)
            for j=1:length(rho)
                % Form intermediate parameters
                Omega = [1,rho(j);rho(j),1];
                pi = eta(i) * ones(K(h),1);
                L = diag([1:K(h)]);
                Lambda = L*L';      % not used to generate data
                
                for b=beta
                    % Data Generating Process
                    U = chol(Omega);            % Cholesky decomposition of Omega=U'U
                    u = V*U;                    % normal random vector with mean=0, var=Omega=U'U
                    z = Z*L';                   % normal random vector with mean=0; var=Lambda=LL'
                    x = z*pi + u(:,2);
                    y = x*b + u(:,1);
    
                    % 2SLS Estimation
                    beta_hat = inv(x'*z*inv(z'*z)*z'*x)*x'*z*inv(z'*z)*z'*y;
                    
                    % Calculate Wald statistic under the null beta0 = 0
                    u_hat = y - x * beta_hat;
                    sigma2_hat = u_hat'*u_hat / n;
                    Wald_hat = (beta_hat - beta0)'*(x'*z*inv(z'*z)*z'*x)*(beta_hat - beta0) / sigma2_hat;

                    % Calculate LMcue statistic under the null beta0 = 0
                    e0 = y - x * beta0;         % vector of y_i - x_i' beta0
                    g = e0 * ones(1,K(h)) .* z; % matrix whose row is g_i(beta0)'
                    g_hat = mean(g)';           % g_hat = z'*e0 / n gives the same value
                    Omega_hat = g'*g / n;
                    H = g * inv(Omega_hat) * g_hat - 1;
                                                % vector of g_i' Omega^(-1) g_hat - 1
                    G = -x * ones(1,K(h)) .* z; % matrix whose row is G_i'
                    D = G' * H / n;
                    LMcue_hat = n * g_hat'*inv(Omega_hat)*D * inv(D'*inv(Omega_hat)*D) * D'*inv(Omega_hat)*g_hat;

                    % Stack estimates to temporary space
                    beta_temp = [beta_temp,beta_hat];
                    Wald_temp = [Wald_temp,Wald_hat];
                    LMcue_temp = [LMcue_temp,LMcue_hat];
                end
            end
        end
    end
    
    % Reshape the result in the form interpreted easily
    beta_temp = reshape(beta_temp, length(beta), length(K)*length(eta)*length(rho))';
    Wald_temp = reshape(Wald_temp, length(beta), length(K)*length(eta)*length(rho))';
    LMcue_temp = reshape(LMcue_temp, length(beta), length(K)*length(eta)*length(rho))';
    
    % Stack estimates for later use
    beta_est(:,:,r) = beta_temp;
    Wald_sim(:,:,r) = Wald_temp;
    LMcue_sim(:,:,r) = LMcue_temp;
end

% Define REJ=1 if the test statistic > 3.84, and REJ=0 otherwise
Wald_rej = Wald_sim > chi2inv(.95,1);
LMcue_rej = LMcue_sim > chi2inv(.95,1);

% Calculate the empirical rejection probability
% Simply take the mean of REJ over the simulation
Wald_rej_prob = mean(Wald_rej,3);
LMcue_rej_prob = mean(LMcue_rej,3);

% Print the result
fprintf('                   [   Wald test power function = Prob(reject the null | beta)   ]\n');
fprintf('---------------||------------------- K=1 ---------------||----------------- K=10 ----------------\n');
fprintf('  (eta / rho)  ||  (.05/.5) (.05/.99)    (1/.5)   (1/.99)  (.05/.5) (.05/.99)    (1/.5)   (1/.99)\n');
fprintf('---------------||--------------------------------------------------------------------------------\n');
fprintf('  beta = %4.1f  ||%10.4f%10.4f%10.4f%10.4f%10.4f%10.4f%10.4f%10.4f\n',[beta;Wald_rej_prob]);
fprintf('\n');
fprintf('                   [  LMcue test power function = Prob(reject the null | beta)   ]\n');
fprintf('---------------||------------------- K=1 ---------------||----------------- K=10 ----------------\n');
fprintf('  (eta / rho)  ||  (.05/.5) (.05/.99)    (1/.5)   (1/.99)  (.05/.5) (.05/.99)    (1/.5)   (1/.99)\n');
fprintf('---------------||--------------------------------------------------------------------------------\n');
fprintf('  beta = %4.1f  ||%10.4f%10.4f%10.4f%10.4f%10.4f%10.4f%10.4f%10.4f\n',[beta;LMcue_rej_prob]);
fprintf('\n');

% Plot the result
figure(1);
CN = 0;             % initialize case number
for h=1:length(K)
    for i=1:length(eta)
        for j=1:length(rho)
            CN = CN + 1;        % increase case number
            subplot(4,2,CN);    % use the panel CN
            plot(beta,Wald_rej_prob(CN,:),'b-',beta,LMcue_rej_prob(CN,:),'r--');
            T = sprintf('K=%d,eta=%1.2f,rho=%1.2f',K(h),eta(i),rho(j));
            title(T);               % display the case
        end
    end
end
legend('Wald rejection prob','LMcue rejection prob');
